pub struct MultiReadTransaction { /* private fields */ }Implementations
Sourceimpl MultiReadTransaction
impl MultiReadTransaction
pub fn new( engine: MultiTransaction, version: Option<CommitVersion>, ) -> Result<Self>
Sourceimpl MultiReadTransaction
impl MultiReadTransaction
pub fn version(&self) -> CommitVersion
pub fn read_as_of_version_exclusive(&mut self, version: CommitVersion)
pub fn read_as_of_version_inclusive(&mut self, version: CommitVersion)
pub fn get(&self, key: &EncodedKey) -> Result<Option<TransactionValue>>
pub fn contains_key(&self, key: &EncodedKey) -> Result<bool>
pub fn scan(&self) -> Result<MultiVersionBatch>
pub fn scan_rev(&self) -> Result<MultiVersionBatch>
pub fn prefix(&self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
pub fn prefix_rev(&self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
Sourcepub fn range(
&self,
range: EncodedKeyRange,
batch_size: usize,
) -> Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>
pub fn range( &self, range: EncodedKeyRange, batch_size: usize, ) -> Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>
Create a streaming iterator for forward range queries.
This properly handles high version density by scanning until batch_size unique logical keys are collected. The stream yields individual entries and maintains cursor state internally.
Sourcepub fn range_rev(
&self,
range: EncodedKeyRange,
batch_size: usize,
) -> Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>
pub fn range_rev( &self, range: EncodedKeyRange, batch_size: usize, ) -> Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>
Create a streaming iterator for reverse range queries.
This properly handles high version density by scanning until batch_size unique logical keys are collected. The stream yields individual entries in reverse key order and maintains cursor state internally.