AdminTransaction

Struct AdminTransaction 

pub struct AdminTransaction {
    pub multi: MultiTransaction,
    pub single: SingleTransaction,
    pub cmd: Option<MultiWriteTransaction>,
    pub event_bus: EventBus,
    pub changes: TransactionalCatalogChanges,
    pub interceptors: Interceptors,
    pub identity: IdentityId,
    /* private fields */
}
An active admin transaction that supports Query + DML + DDL operations.

AdminTransaction is the most privileged transaction type, capable of executing DDL (shape changes), DML (data mutations), and queries. It tracks catalog definition changes (TransactionalCatalogChanges) for DDL.

The transaction will auto-rollback on drop if not explicitly committed.

Fields

multi: MultiTransactionsingle: SingleTransactioncmd: Option<MultiWriteTransaction>event_bus: EventBuschanges: TransactionalCatalogChangesinterceptors: Interceptorsidentity: IdentityId

The identity executing this transaction.

Implementations

Source

impl AdminTransaction

pub fn new( multi: MultiTransaction, single: SingleTransaction, event_bus: EventBus, interceptors: Interceptors, identity: IdentityId, clock: Clock, ) -> Result<Self>

Creates a new active admin transaction with a pre-commit callback

pub fn set_executor(&mut self, executor: Arc<dyn RqlExecutor>)

Set the RQL executor for this transaction.

pub fn rql(&mut self, rql: &str, params: Params) -> ExecutionResult

Execute RQL within this transaction using the attached executor.

Panics if no RqlExecutor has been set on this transaction.

pub fn event_bus(&self) -> &EventBus

pub fn commit(&mut self) -> Result<CommitVersion>

Commit the transaction. Since single transactions are short-lived and auto-commit, this only commits the multi transaction.

pub fn rollback(&mut self) -> Result<()>

Rollback the transaction.

pub fn pending_writes(&self) -> &PendingWrites

Get access to the pending writes in this transaction

pub fn with_single_query<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>
where I: IntoIterator<Item = &'a EncodedKey> + Send, F: FnOnce(&mut SingleReadTransaction<'_>) -> Result<R> + Send, R: Send,

Execute a function with query access to the single transaction.

pub fn with_single_command<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>
where I: IntoIterator<Item = &'a EncodedKey> + Send, F: FnOnce(&mut SingleWriteTransaction<'_>) -> Result<R> + Send, R: Send,

Execute a function with query access to the single transaction.

pub fn with_multi_query<F, R>(&self, f: F) -> Result<R>
where F: FnOnce(&mut QueryTransaction) -> Result<R>,

Execute a function with a query transaction view.

pub fn with_multi_query_as_of_exclusive<F, R>( &self, version: CommitVersion, f: F, ) -> Result<R>
where F: FnOnce(&mut QueryTransaction) -> Result<R>,

pub fn with_multi_query_as_of_inclusive<F, R>( &self, version: CommitVersion, f: F, ) -> Result<R>
where F: FnOnce(&mut QueryTransaction) -> Result<R>,

pub fn begin_single_query<'a, I>( &self, keys: I, ) -> Result<SingleReadTransaction<'_>>
where I: IntoIterator<Item = &'a EncodedKey>,

Begin a single-version query transaction for specific keys

pub fn begin_single_command<'a, I>( &self, keys: I, ) -> Result<SingleWriteTransaction<'_>>
where I: IntoIterator<Item = &'a EncodedKey>,

Begin a single-version command transaction for specific keys

pub fn get_changes(&self) -> &TransactionalCatalogChanges

Get reference to catalog changes for this transaction

pub fn track_row_change(&mut self, change: RowChange)

Track a row change for post-commit event emission

pub fn track_flow_change(&mut self, change: Change)

Track a flow change for transactional view pre-commit processing.

pub fn version(&self) -> CommitVersion

Get the transaction version

pub fn id(&self) -> TransactionId

Get the transaction ID

pub fn get(&mut self, key: &EncodedKey) -> Result<Option<MultiVersionRow>>

Get a value by key

pub fn get_committed( &mut self, key: &EncodedKey, ) -> Result<Option<MultiVersionRow>>

Read the committed value at the transaction’s read version, ignoring pending intra-tx writes.

pub fn contains_key(&mut self, key: &EncodedKey) -> Result<bool>

Check if a key exists

pub fn prefix(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>

Get a prefix batch

pub fn prefix_rev(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>

Get a reverse prefix batch

pub fn read_as_of_version_exclusive( &mut self, version: CommitVersion, ) -> Result<()>

Read as of version exclusive

pub fn set(&mut self, key: &EncodedKey, row: EncodedRow) -> Result<()>

Set a key-value pair

pub fn unset(&mut self, key: &EncodedKey, row: EncodedRow) -> Result<()>

Unset a key, preserving the deleted values.

pub fn remove(&mut self, key: &EncodedKey) -> Result<()>

Remove a key without preserving the deleted values.

pub fn mark_preexisting(&mut self, key: &EncodedKey) -> Result<()>

pub fn range( &mut self, range: EncodedKeyRange, batch_size: usize, ) -> Result<Box<dyn Iterator<Item = Result<MultiVersionRow>> + Send + '_>>

Create a streaming iterator for forward range queries.

pub fn range_rev( &mut self, range: EncodedKeyRange, batch_size: usize, ) -> Result<Box<dyn Iterator<Item = Result<MultiVersionRow>> + Send + '_>>

Create a streaming iterator for reverse range queries.