pub struct QueryTransaction {
pub identity: IdentityId,
/* private fields */
}Expand description
An active query transaction that holds a multi query transaction and provides query-only access to single storage.
Fields
identity: IdentityIdThe identity executing this transaction.
Implementations
Sourceimpl QueryTransaction
impl QueryTransaction
Sourcepub fn new(
multi: MultiReadTransaction,
single: SingleTransaction,
identity: IdentityId,
) -> Self
pub fn new( multi: MultiReadTransaction, single: SingleTransaction, identity: IdentityId, ) -> Self
Creates a new active query transaction
Sourcepub fn set_executor(&mut self, executor: Arc<dyn RqlExecutor>)
pub fn set_executor(&mut self, executor: Arc<dyn RqlExecutor>)
Set the RQL executor for this transaction.
Sourcepub fn rql(&mut self, rql: &str, params: Params) -> ExecutionResult
pub fn rql(&mut self, rql: &str, params: Params) -> ExecutionResult
Execute RQL within this transaction using the attached executor.
Panics if no RqlExecutor has been set on this transaction.
Sourcepub fn version(&self) -> CommitVersion
pub fn version(&self) -> CommitVersion
Get the transaction version
Sourcepub fn id(&self) -> TransactionId
pub fn id(&self) -> TransactionId
Get the transaction ID
Sourcepub fn get(&mut self, key: &EncodedKey) -> Result<Option<MultiVersionRow>>
pub fn get(&mut self, key: &EncodedKey) -> Result<Option<MultiVersionRow>>
Get a value by key
Sourcepub fn contains_key(&mut self, key: &EncodedKey) -> Result<bool>
pub fn contains_key(&mut self, key: &EncodedKey) -> Result<bool>
Check if a key exists
Sourcepub fn prefix(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
pub fn prefix(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
Get a prefix batch
Sourcepub fn prefix_rev(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
pub fn prefix_rev(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
Get a reverse prefix batch
Sourcepub fn read_as_of_version_exclusive(
&mut self,
version: CommitVersion,
) -> Result<()>
pub fn read_as_of_version_exclusive( &mut self, version: CommitVersion, ) -> Result<()>
Read as of version exclusive
Sourcepub fn range(
&self,
range: EncodedKeyRange,
batch_size: usize,
) -> Box<dyn Iterator<Item = Result<MultiVersionRow>> + Send + '_>
pub fn range( &self, range: EncodedKeyRange, batch_size: usize, ) -> Box<dyn Iterator<Item = Result<MultiVersionRow>> + Send + '_>
Create a streaming iterator for forward range queries.
Sourcepub fn range_rev(
&self,
range: EncodedKeyRange,
batch_size: usize,
) -> Box<dyn Iterator<Item = Result<MultiVersionRow>> + Send + '_>
pub fn range_rev( &self, range: EncodedKeyRange, batch_size: usize, ) -> Box<dyn Iterator<Item = Result<MultiVersionRow>> + Send + '_>
Create a streaming iterator for reverse range queries.
Sourcepub fn with_single_query<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>where
I: IntoIterator<Item = &'a EncodedKey> + Send,
F: FnOnce(&mut SingleReadTransaction<'_>) -> Result<R> + Send,
R: Send,
pub fn with_single_query<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>where
I: IntoIterator<Item = &'a EncodedKey> + Send,
F: FnOnce(&mut SingleReadTransaction<'_>) -> Result<R> + Send,
R: Send,
Execute a function with query access to the single transaction.
Sourcepub fn with_multi_query<F, R>(&mut self, f: F) -> Result<R>
pub fn with_multi_query<F, R>(&mut self, f: F) -> Result<R>
Execute a function with access to the multi query transaction. This operates within the same transaction context.
Sourcepub fn begin_single_query<'a, I>(
&self,
keys: I,
) -> Result<SingleReadTransaction<'_>>where
I: IntoIterator<Item = &'a EncodedKey>,
pub fn begin_single_query<'a, I>(
&self,
keys: I,
) -> Result<SingleReadTransaction<'_>>where
I: IntoIterator<Item = &'a EncodedKey>,
Begin a single-version query transaction for specific keys