pub struct AdminTransaction {
pub multi: MultiTransaction,
pub single: SingleTransaction,
pub cmd: Option<MultiWriteTransaction>,
pub event_bus: EventBus,
pub changes: TransactionalDefChanges,
pub testing: Option<TestingContext>,
/* private fields */
}Expand description
An active admin transaction that supports Query + DML + DDL operations.
AdminTransaction is the most privileged transaction type, capable of executing DDL (schema changes), DML (data mutations), and queries. It tracks catalog definition changes (TransactionalDefChanges) for DDL.
The transaction will auto-rollback on drop if not explicitly committed.
Fields
multi: MultiTransactionsingle: SingleTransactioncmd: Option<MultiWriteTransaction>event_bus: EventBuschanges: TransactionalDefChangestesting: Option<TestingContext>Testing audit log. Set by the VM when in test context.
Implementations
Sourceimpl AdminTransaction
impl AdminTransaction
Sourcepub fn restore_savepoint(&mut self, sp: Savepoint)
pub fn restore_savepoint(&mut self, sp: Savepoint)
Restore transaction state to a previously created savepoint.
Sourcepub fn clear_test_flow_state(&mut self)
pub fn clear_test_flow_state(&mut self)
Reset test-only flow bookkeeping so setup statements do not appear as
in-test mutations when RUN TESTS later performs an inline flush.
Sourcepub fn capture_testing_pre_commit<F>(&mut self, f: F) -> Result<()>
pub fn capture_testing_pre_commit<F>(&mut self, f: F) -> Result<()>
Execute test-only pre-commit style processing without committing.
This is used by testing helpers that need commit-time flow work materialized while still staying inside the test savepoint.
Sourceimpl AdminTransaction
impl AdminTransaction
Sourcepub fn new(
multi: MultiTransaction,
single: SingleTransaction,
event_bus: EventBus,
interceptors: Interceptors,
) -> Result<Self>
pub fn new( multi: MultiTransaction, single: SingleTransaction, event_bus: EventBus, interceptors: Interceptors, ) -> Result<Self>
Creates a new active admin transaction with a pre-commit callback
pub fn event_bus(&self) -> &EventBus
Sourcepub fn commit(&mut self) -> Result<CommitVersion>
pub fn commit(&mut self) -> Result<CommitVersion>
Commit the transaction. Since single transactions are short-lived and auto-commit, this only commits the multi transaction.
Sourcepub fn pending_writes(&self) -> &PendingWrites
pub fn pending_writes(&self) -> &PendingWrites
Get access to the pending writes in this transaction
Sourcepub fn with_single_query<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>where
I: IntoIterator<Item = &'a EncodedKey> + Send,
F: FnOnce(&mut SingleReadTransaction<'_>) -> Result<R> + Send,
R: Send,
pub fn with_single_query<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>where
I: IntoIterator<Item = &'a EncodedKey> + Send,
F: FnOnce(&mut SingleReadTransaction<'_>) -> Result<R> + Send,
R: Send,
Execute a function with query access to the single transaction.
Sourcepub fn with_single_command<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>where
I: IntoIterator<Item = &'a EncodedKey> + Send,
F: FnOnce(&mut SingleWriteTransaction<'_>) -> Result<R> + Send,
R: Send,
pub fn with_single_command<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>where
I: IntoIterator<Item = &'a EncodedKey> + Send,
F: FnOnce(&mut SingleWriteTransaction<'_>) -> Result<R> + Send,
R: Send,
Execute a function with query access to the single transaction.
Sourcepub fn with_multi_query<F, R>(&self, f: F) -> Result<R>
pub fn with_multi_query<F, R>(&self, f: F) -> Result<R>
Execute a function with a query transaction view.
pub fn with_multi_query_as_of_exclusive<F, R>( &self, version: CommitVersion, f: F, ) -> Result<R>
pub fn with_multi_query_as_of_inclusive<F, R>( &self, version: CommitVersion, f: F, ) -> Result<R>
Sourcepub fn begin_single_query<'a, I>(
&self,
keys: I,
) -> Result<SingleReadTransaction<'_>>where
I: IntoIterator<Item = &'a EncodedKey>,
pub fn begin_single_query<'a, I>(
&self,
keys: I,
) -> Result<SingleReadTransaction<'_>>where
I: IntoIterator<Item = &'a EncodedKey>,
Begin a single-version query transaction for specific keys
Sourcepub fn begin_single_command<'a, I>(
&self,
keys: I,
) -> Result<SingleWriteTransaction<'_>>where
I: IntoIterator<Item = &'a EncodedKey>,
pub fn begin_single_command<'a, I>(
&self,
keys: I,
) -> Result<SingleWriteTransaction<'_>>where
I: IntoIterator<Item = &'a EncodedKey>,
Begin a single-version command transaction for specific keys
Sourcepub fn get_changes(&self) -> &TransactionalDefChanges
pub fn get_changes(&self) -> &TransactionalDefChanges
Get reference to catalog changes for this transaction
Sourcepub fn track_row_change(&mut self, change: RowChange)
pub fn track_row_change(&mut self, change: RowChange)
Track a row change for post-commit event emission
Sourcepub fn track_flow_change(&mut self, change: Change)
pub fn track_flow_change(&mut self, change: Change)
Track a flow change for transactional view pre-commit processing
Sourcepub fn version(&self) -> CommitVersion
pub fn version(&self) -> CommitVersion
Get the transaction version
Sourcepub fn id(&self) -> TransactionId
pub fn id(&self) -> TransactionId
Get the transaction ID
Sourcepub fn get(&mut self, key: &EncodedKey) -> Result<Option<MultiVersionValues>>
pub fn get(&mut self, key: &EncodedKey) -> Result<Option<MultiVersionValues>>
Get a value by key
Sourcepub fn contains_key(&mut self, key: &EncodedKey) -> Result<bool>
pub fn contains_key(&mut self, key: &EncodedKey) -> Result<bool>
Check if a key exists
Sourcepub fn prefix(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
pub fn prefix(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
Get a prefix batch
Sourcepub fn prefix_rev(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
pub fn prefix_rev(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>
Get a reverse prefix batch
Sourcepub fn read_as_of_version_exclusive(
&mut self,
version: CommitVersion,
) -> Result<()>
pub fn read_as_of_version_exclusive( &mut self, version: CommitVersion, ) -> Result<()>
Read as of version exclusive
Sourcepub fn set(&mut self, key: &EncodedKey, row: EncodedValues) -> Result<()>
pub fn set(&mut self, key: &EncodedKey, row: EncodedValues) -> Result<()>
Set a key-value pair
Sourcepub fn unset(&mut self, key: &EncodedKey, values: EncodedValues) -> Result<()>
pub fn unset(&mut self, key: &EncodedKey, values: EncodedValues) -> Result<()>
Unset a key, preserving the deleted values.
Sourcepub fn remove(&mut self, key: &EncodedKey) -> Result<()>
pub fn remove(&mut self, key: &EncodedKey) -> Result<()>
Remove a key without preserving the deleted values.
Sourcepub fn range(
&mut self,
range: EncodedKeyRange,
batch_size: usize,
) -> Result<Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>>
pub fn range( &mut self, range: EncodedKeyRange, batch_size: usize, ) -> Result<Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>>
Create a streaming iterator for forward range queries.
Sourcepub fn range_rev(
&mut self,
range: EncodedKeyRange,
batch_size: usize,
) -> Result<Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>>
pub fn range_rev( &mut self, range: EncodedKeyRange, batch_size: usize, ) -> Result<Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>>
Create a streaming iterator for reverse range queries.