Transaction

Enum Transaction 

pub enum Transaction<'a> {
    Command(&'a mut CommandTransaction),
    Admin(&'a mut AdminTransaction),
    Query(&'a mut QueryTransaction),
    Subscription(&'a mut SubscriptionTransaction),
}
An enum that can hold either a command, admin, query, or subscription transaction for flexible execution

Variants



Command(&'a mut CommandTransaction)



Admin(&'a mut AdminTransaction)



Query(&'a mut QueryTransaction)



Subscription(&'a mut SubscriptionTransaction)

Implementations

Source

impl<'a> Transaction<'a>

pub fn version(&self) -> CommitVersion

Get the transaction version

pub fn id(&self) -> TransactionId

Get the transaction ID

pub fn get(&mut self, key: &EncodedKey) -> Result<Option<MultiVersionValues>>

Get a value by key (async method)

pub fn contains_key(&mut self, key: &EncodedKey) -> Result<bool>

Check if a key exists (async method)

pub fn prefix(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>

Get a prefix batch (async method)

pub fn prefix_rev(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>

Get a reverse prefix batch (async method)

pub fn read_as_of_version_exclusive( &mut self, version: CommitVersion, ) -> Result<()>

Read as of version exclusive (async method)

pub fn range( &mut self, range: EncodedKeyRange, batch_size: usize, ) -> Result<Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>>

Create a streaming iterator for forward range queries.

pub fn range_rev( &mut self, range: EncodedKeyRange, batch_size: usize, ) -> Result<Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>>

Create a streaming iterator for reverse range queries.

Source

impl<'a> Transaction<'a>

pub fn reborrow(&mut self) -> Transaction<'_>

Re-borrow this transaction with a shorter lifetime, enabling multiple sequential uses of the same transaction binding.

pub fn command(self) -> &'a mut CommandTransaction

Extract the underlying CommandTransaction, panics if this is not a Command transaction

pub fn admin(self) -> &'a mut AdminTransaction

Extract the underlying AdminTransaction, panics if this is not an Admin transaction

pub fn query(self) -> &'a mut QueryTransaction

Extract the underlying QueryTransaction, panics if this is not a Query transaction

pub fn subscription(self) -> &'a mut SubscriptionTransaction

Extract the underlying SubscriptionTransaction, panics if this is not a Subscription transaction

pub fn command_mut(&mut self) -> &mut CommandTransaction

Get a mutable reference to the underlying CommandTransaction, panics if this is not a Command transaction

pub fn admin_mut(&mut self) -> &mut AdminTransaction

Get a mutable reference to the underlying AdminTransaction, panics if this is not an Admin transaction

pub fn query_mut(&mut self) -> &mut QueryTransaction

Get a mutable reference to the underlying QueryTransaction, panics if this is not a Query transaction

pub fn subscription_mut(&mut self) -> &mut SubscriptionTransaction

Get a mutable reference to the underlying SubscriptionTransaction, panics if this is not a Subscription transaction

pub fn begin_single_query<'b, I>( &self, keys: I, ) -> Result<SingleReadTransaction<'_>>
where I: IntoIterator<Item = &'b EncodedKey>,

Begin a single-version query transaction for specific keys

pub fn begin_single_command<'b, I>( &self, keys: I, ) -> Result<SingleWriteTransaction<'_>>
where I: IntoIterator<Item = &'b EncodedKey>,

Begin a single-version write transaction for specific keys. Panics on Query transactions.

pub fn set(&mut self, key: &EncodedKey, row: EncodedValues) -> Result<()>

Set a key-value pair. Panics on Query transactions.

pub fn unset(&mut self, key: &EncodedKey, values: EncodedValues) -> Result<()>

Unset (delete with tombstone) a key-value pair. Panics on Query transactions.

pub fn remove(&mut self, key: &EncodedKey) -> Result<()>

Remove a key. Panics on Query transactions.

pub fn track_row_change(&mut self, change: RowChange)

Track a row change for post-commit event emission. Panics on Query transactions.

pub fn track_flow_change(&mut self, change: Change)

Track a flow change for transactional view pre-commit processing. Panics on Query transactions.