QueryTransaction

Struct QueryTransaction 

pub struct QueryTransaction { /* private fields */ }
An active query transaction that holds a multi query transaction and provides query-only access to single storage.

Implementations

Source

impl QueryTransaction

pub fn new(multi: MultiReadTransaction, single: SingleTransaction) -> Self

Creates a new active query transaction

pub fn version(&self) -> CommitVersion

Get the transaction version

pub fn id(&self) -> TransactionId

Get the transaction ID

pub fn get(&mut self, key: &EncodedKey) -> Result<Option<MultiVersionValues>>

Get a value by key

pub fn contains_key(&mut self, key: &EncodedKey) -> Result<bool>

Check if a key exists

pub fn prefix(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>

Get a prefix batch

pub fn prefix_rev(&mut self, prefix: &EncodedKey) -> Result<MultiVersionBatch>

Get a reverse prefix batch

pub fn read_as_of_version_exclusive( &mut self, version: CommitVersion, ) -> Result<()>

Read as of version exclusive

pub fn range( &self, range: EncodedKeyRange, batch_size: usize, ) -> Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>

Create a streaming iterator for forward range queries.

pub fn range_rev( &self, range: EncodedKeyRange, batch_size: usize, ) -> Box<dyn Iterator<Item = Result<MultiVersionValues>> + Send + '_>

Create a streaming iterator for reverse range queries.

pub fn with_single_query<'a, I, F, R>(&self, keys: I, f: F) -> Result<R>
where I: IntoIterator<Item = &'a EncodedKey> + Send, F: FnOnce(&mut SingleReadTransaction<'_>) -> Result<R> + Send, R: Send,

Execute a function with query access to the single transaction.

pub fn with_multi_query<F, R>(&mut self, f: F) -> Result<R>
where F: FnOnce(&mut MultiReadTransaction) -> Result<R>,

Execute a function with access to the multi query transaction. This operates within the same transaction context.

pub fn begin_single_query<'a, I>( &self, keys: I, ) -> Result<SingleReadTransaction<'_>>
where I: IntoIterator<Item = &'a EncodedKey>,

Begin a single-version query transaction for specific keys